/*
   dense.c
   San Skulrattanakulchai
   11/11/99
 */
#include <stdlib.h>   /* for drand48() */
#include "digraph.h"
#include "gen.h"
#include "random.h"

/*
   Generate a random dense graph of nv vertices.
   Let d = nv*(nv-1) / oneOverp.
   If expected is true, then generate a digraph with d edges on average.
   Otherwise generate a digraph with exactly d edges using Selection Sampling.
 */
DigraphPtr dense(int nv, int oneOverp, bool expected)
{
  DigraphPtr Gp;
  Vertex v, w;
  int d, t, m, N;
  double U;

  makeEmptyDigraph(nv, Gp);

  if (expected) {
    for (v = 1; v <= nv; v++)
      for (w = 1; w <= nv; w++) {
        if (v != w && randNum(1, oneOverp) == 1)
          addEdge(v, w, Gp);
      }
  } else {
    N = nv*(nv-1);
    d = N / oneOverp;
    t = m = 0;
    for (v = 1; v <= nv; v++) {
      for (w = 1; w <= nv; w++) {
        if (v == w)
          continue;
        U = drand48();
        if ((N-t)* U < d-m) {
          addEdge(v, w, Gp);
          if (++m == d)
            return Gp;
        }
        t++;
      }
    }
  }

  return Gp;
}
