/*
   gscc1.C
   San Skulrattanakulchai
   11/11/99
   Implementation of Gabow's strong component algorithm using path-based dfs.
   Use variables for top of boundary stack, array and structure accesses.
 */
#include "digraph.h"
#include "gscc.h"
#include "intarray.h"
#include "stack.h"

static int gc;   /* component number */
static Stack S;
static Stack B;
static int btb;

int gstrong1(DigraphPtr Gp, IntArray I)
{
  Vertex v;
  int nv;

  gc = nv = Gp->n;
  makeStack(nv, S); makeStack(nv, B);

  for (v = 1; v <= nv; v++)
    if (I[v] == 0)
      gdfs1(v, Gp, I);

  deleteStack(B); deleteStack(S);

  return gc - nv;
}

void gdfs1(Vertex v, DigraphPtr Gp, IntArray I)
{
  NodePtr p;
  Vertex w;

  push(v, S); btb = I[v] = top(S); push(btb,  B); /* add v to the end of P */

  for (p = Gp->adj[v]; p; p = p->next)
    if (I[w = p->vertex] == 0)
      gdfs1(w, Gp, I);
    else if (btb > I[w]) /* contract vertices */
      popWhileGreater(I[w], B, btb);

  if (btb == I[v]) {
    /* number vertices of the next strong component */
    voidPop(B);
    btb = B[top(B)];
    gc++;
    while (top(S) >= I[v])
      I[pop(S)] = gc;
  }
}
