/*
   tester.c
   San Skulrattanakulchai
   11/11/99
 */
#include <stdio.h>        /* for  printf() */
#include <time.h>         /* for clock(), clock_t, CLOCK_PER_SEC */
#include "digraph.h"
#include "gscc.h"
#include "intarray.h"
#include "kscc.h"
#include "tscc.h"
#include "tester.h"
#include "util.h"

void printHead()
{
  printf("%8s%8s%8s%8s%8s%8s%8s%8s%8s%8s\n",
    "#nodes", "#arcs", "#SCC's",
    "path.0", "path.1",
    "LWPT.0", "LWPT.1", "LWPT.2",
    "2P.0", "2P.1");
}

void testDigraphs(DigraphPtr Gp)
{
  int nv = Gp->n;
  IntArray I, I1, C, C1, C2, K, K1;
  int gcomp, gcomp1, tcomp, tcomp1, tcomp2, kcomp, kcomp1;
  clock_t begin, mid1, mid2, mid3, mid4, mid5, mid6, end;
  float gtime, gtime1, ttime, ttime1, ttime2, ktime, ktime1;

  begin = clock();
  makeIntArray(nv, I); gcomp = gstrong(Gp, I);
  mid1 = clock();
  makeIntArray(nv, I1); gcomp1 = gstrong1(Gp, I1);
  mid2 = clock();
  makeIntArray(nv, C); tcomp = tstrong(Gp, C);
  mid3 = clock();
  makeIntArray(nv, C1); tcomp1 = tstrong1(Gp, C1);
  mid4 = clock();
  makeIntArray(nv, C2); tcomp2 = tstrong2(Gp, C2);
  mid5 = clock();
  makeIntArray(nv, K); kcomp = kstrong(Gp, K);
  mid6 = clock();
  makeIntArray(nv, K1); kcomp1 = kstrong1(Gp, K1);
  end = clock();

  if (!(gcomp == gcomp1 && gcomp1 == tcomp && tcomp == tcomp1 &&
    tcomp1 == tcomp2 && tcomp2 == kcomp && kcomp == kcomp1))
    fatal("Some different algs give different #components.");

#ifdef DEBUG
{
  bool errCode1, errCode2, errCode3, errCode4;
  IntArray imap, cmap;
  Vertex v;

  errCode1 = errCode2 = errCode3 = errCode4 = false;
  makeIntArray(kcomp, imap); makeIntArray(kcomp, cmap);
  for (v = 1; v <= nv; v++) {
    if (I[v] != C[v])
      errCode1 = true;
    if (imap[K[v]-nv] == 0)
      imap[K[v]-nv] = I[v];
    else if (imap[K[v]-nv] != I[v])
      errCode2 = true;
    if (cmap[K[v]-nv] == 0)
      cmap[K[v]-nv] = C[v];
    else if (cmap[K[v]-nv] != C[v])
      errCode3 = true;
    if (!(I[v]==I1[v] && C[v]==C1[v] && C[v]==C2[v] && K[v]==K1[v]))
      errCode4 = true;
  }
  deleteIntArray(cmap); deleteIntArray(imap);

  if (errCode1)
    printf("Gabow disagrees with Tarjan.\n");
  if (errCode2)
    printf("Gabow disagrees with K&S.\n");
  if (errCode3)
    printf("Tarjan disagrees with K&S.\n");
  if (errCode1 || errCode2 || errCode3)
    fatal("Some of gscc(), tscc(), or kscc() is incorrect.");
  if (errCode4)
    fatal("Some of the modified strong component alg's is incorrect.");
}
#endif

  gtime = (float) (mid1 - begin) / CLOCKS_PER_SEC;
  gtime1 = (float) (mid2 - mid1) / CLOCKS_PER_SEC;
  ttime = (float) (mid3 - mid2) / CLOCKS_PER_SEC;
  ttime1 = (float) (mid4 - mid3) / CLOCKS_PER_SEC;
  ttime2 = (float) (mid5 - mid4) / CLOCKS_PER_SEC;
  ktime = (float) (mid6 - mid5) / CLOCKS_PER_SEC;
  ktime1 = (float) (end - mid6) / CLOCKS_PER_SEC;

  printf("%8d%8d%8d%8.2f%8.2f%8.2f%8.2f%8.2f%8.2f%8.2f\n",
    nv, Gp->m, gcomp, gtime, gtime1, ttime, ttime1, ttime2, ktime, ktime1);

  deleteIntArray(K1); deleteIntArray(K);
  deleteIntArray(C2); deleteIntArray(C1); deleteIntArray(C);
  deleteIntArray(I1); deleteIntArray(I);
}
