/*
   tscc.C
   San Skulrattanakulchai
   11/11/99
   Implemnentation of Tarjan's strong component algorithm using LOWLINK values.
 */
#include "digraph.h"
#include "intarray.h"
#include "stack.h"
#include "tscc.h"

static int dtime;         /* discovery time */
static int tc;            /* component number */
static Stack S;           /* stack of vertices in current component */
static IntArray Number;   /* array of discovery times */
static IntArray Lowlink;  /* array of Lowlink values */
static IntArray Onstack;  /* Onstack[v] = 1,  if v is on S */
                          /*            = 0,  otherwise.   */

int tstrong(DigraphPtr Gp, IntArray C)
{
  Vertex v;

  makeStack(Gp->n, S);
  makeIntArray(Gp->n, Number);
  makeIntArray(Gp->n, Onstack);
  makeIntArray(Gp->n, Lowlink);

  tc = Gp->n;
  dtime = 0;
  for (v = 1; v <= Gp->n; v++)
    if (Number[v] == 0)  /* v is not yet numbered */
      tdfs(v, Gp, C);

  deleteIntArray(Lowlink);
  deleteIntArray(Onstack);
  deleteIntArray(Number);
  deleteStack(S);

  return tc - Gp->n;
}

void tdfs(Vertex v, DigraphPtr Gp, IntArray C)
{
  NodePtr p;
  Vertex w;

  Lowlink[v] = Number[v] = ++dtime;
  push(v, S); Onstack[v] = 1;
  for (p = Gp->adj[v]; p; p = p->next)
    if (Number[w = p->vertex] == 0) {  /* w is not yet numbered */
      tdfs(w, Gp, C);
      if (Lowlink[w] < Lowlink[v])
	Lowlink[v] = Lowlink[w];
    } else if (Number[w] < Number[v] && Onstack[w] && Number[w] < Lowlink[v])
      Lowlink[v] = Number[w];

  if (Lowlink[v] == Number[v]) {
    tc++;
    while (Number[S[top(S)]] >= Number[v]) {
      Onstack[w = pop(S)] = 0;
      C[w] = tc;
    }
  }
}
