/*
   tscc1.C
   San Skulrattanakulchai
   11/11/99
   Implemnentation of Tarjan's strong component algorithm.
   Eliminate Onstack[], Number[], and use the following definition for C[]
                  
         C[v] =   0,  if v is undiscovered
                  d,  if v's discovery time is d and v is on the stack
                  c,  if v's component is c and v is off the stack
 */
#include "digraph.h"
#include "intarray.h"
#include "stack.h"
#include "tscc.h"

static int dtime;         /* discovery time */
static int tc;            /* component number */
static Stack S;           /* stack of vertices in current component */
static IntArray Lowlink;  /* array of Lowlink values */

int tstrong1(DigraphPtr Gp, IntArray C)
{
  Vertex v;

  makeStack(Gp->n, S); makeIntArray(Gp->n, Lowlink);

  tc = Gp->n;
  dtime = 0;
  for (v = 1; v <= Gp->n; v++)
    if (C[v] == 0)  // v is not yet numbered
      tdfs1(v, Gp, C);

  deleteIntArray(Lowlink); deleteStack(S);

  return tc - Gp->n;
}

void tdfs1(Vertex v, DigraphPtr Gp, IntArray C)
{
  NodePtr p;
  Vertex w;

  Lowlink[v] = C[v] = ++dtime;
  push(v, S);
  for (p = Gp->adj[v]; p; p = p->next)
    if (C[w = p->vertex] == 0) {  /* w is not yet numbered */
      tdfs1(w, Gp, C);
      if (Lowlink[w] < Lowlink[v])
	Lowlink[v] = Lowlink[w];
    } else if (C[w] < Lowlink[v])
      Lowlink[v] = C[w];

  if (Lowlink[v] == C[v]) {
    tc++;
    do
      C[w = pop(S)] = tc;
    while (w != v);
  }
}
