/*
   tscc2.C
   San Skulrattanakulchai
   11/11/99
   Implemnentation of Tarjan's strong component algorithm.
   (1) Eliminate Onstack[], Number[], and use the following definition for C[]
                  
         C[v] =   0,  if v is undiscovered
                  d,  if v's discovery time is d and v is on the stack
                  c,  if v's component is c and v is off the stack
  
   (2) Incorporate variables for Lowlink[v], Gp->n
 */
#include "digraph.h"
#include "intarray.h"
#include "stack.h"
#include "tscc.h"

static int dtime;         /* discovery time */
static int tc;            /* component number */
static Stack S;           /* stack of vertices in current component */
static IntArray Lowlink;  /* array of Lowlink values */

int tstrong2(DigraphPtr Gp, IntArray C)
{
  Vertex v;
  int nv;

  nv = tc = Gp->n;
  makeStack(nv, S); makeIntArray(nv, Lowlink);

  dtime = 0;
  for (v = 1; v <= nv; v++)
    if (C[v] == 0)  /* v is not yet numbered */
      tdfs2(v, Gp, C);

  deleteIntArray(Lowlink); deleteStack(S);

  return tc - nv;
}

void tdfs2(Vertex v, DigraphPtr Gp, IntArray C)
{
  NodePtr p;
  Vertex w;
  int lwlnkv;   /* a temporary for holding the value of Lowlink[v] */

  lwlnkv = C[v] = ++dtime;
  push(v, S);
  for (p = Gp->adj[v]; p; p = p->next)
    if (C[w = p->vertex] == 0) {  /* w is not yet numbered */
      tdfs2(w, Gp, C);
      if (Lowlink[w] < lwlnkv)
	lwlnkv = Lowlink[w];
    } else if (C[w] < lwlnkv)
      lwlnkv = C[w];

  if ((Lowlink[v]=lwlnkv) == C[v]) {
    tc++;
    do
      C[w = pop(S)] = tc;
    while (w != v);
  }
}
